@@ -54,6 +54,7 @@ EXEC_ON_PAL_STATE_NLN= ''
 EXEC_ON_PAL_STATE_CHANGE= ''
 EXEC_ON_PAL_CONNECTED= ''
 EXEC_ON_PAL_DISCONNECTED= ''
+EXEC_ON_NEW_MAIL= ''
 #################
 
 PROG_NAME= 'Pebrot'
@@ -88,6 +89,7 @@ HELPMSG= _(r'''Commands begin with '\' or '/', everything else are messages.
     \a user     Add user to buddy list.
     \b user     Block user.
     \r user     Remove user from buddy list.
+    \o user     Open chat window with a user not on your buddy list.
     \s state    Change state. state must be one of the following:
                   nln: Online          fln: Offline        hdn: Appear Offline
                   idl: Idle            awy: Away           bsy: Busy
@@ -804,7 +806,11 @@ class TextMSN(MSN.Session):
             i= i + 1    
 
         #self.pMain( (LISTLINE  % ('-' * len(title))) + '\n' )
-        self.pWin( chat, (LISTLINE  % ('-' * len(title))) + '\n', refresh= 0
+        if TIMESTAMP:
+            self.pWin( chat, '------------= %s =------------\n' % time.strftime(TIMESTAMP_FORMAT
+                                         , time.localtime()), 'TIMESTAMP' )
+        else:
+            self.pWin( chat, (LISTLINE  % ('-' * len(title))) + '\n', refresh= 0
                     , color= 'LIST_DELIM' )
         self.refreshWin()
         self.Text.refresh()
@@ -857,6 +863,30 @@ class TextMSN(MSN.Session):
         if len(parts) == 0:
             return
         
+		    
+        # Added to allow Pebrot to remain offline
+	if self.state == 'FLN':
+            if parts[0][1:] == 'q':
+                sys.exit()
+            elif parts[0][1:] == 's':
+                if len(parts)<2:
+                    self.pInfo( _('state: %s') % MSN.STATES[self.state] )
+                else:
+                    state= upper( parts[1] )
+                    if state != 'FLN' and MSN.STATES.has_key( state ):
+                    	self.state = state
+                    	self.reconnect()
+                        self.idle= 0
+                        #self.lastStartedTyping= time.time()
+                        #self.alarmExecAndProgramNext()
+                    else:
+                        self.helpInfo()
+		return
+	    else:
+                self.pError( _('Error: Must be online to send commands or messages.') )
+	        return
+        # End Edit
+
         if parts[0][0] in ['\\', '/']:
             command= parts[0][1:]
             self.pDebug( 'com=' + command )
@@ -912,7 +942,12 @@ class TextMSN(MSN.Session):
 
                 allUsers= self.usersList.keys()
                 allUsers.sort()
-                user= allUsers[int(parts[1])]
+                nUser = int(parts[1])
+
+                if (nUser<0 or nUser>=len(allUsers)):
+                    self.pError( _('User with this number does not exist') )
+                    return
+                user= allUsers[nUser]
                 self.activeChat.invite( user )
 
             elif command == 'c':
@@ -957,6 +992,31 @@ class TextMSN(MSN.Session):
             elif command == 'u':
                 if self.activeChat != self.sortedChats[0]:
                     self.showChatUsers( self.activeChat )
+
+            elif command == 'o':
+                if len(parts)<2:
+                    self.helpInfo()
+                else:
+                    if self.state in ['HDN', 'FLN']:
+                        self.pError( _('Error: Must be online for talking') )
+                        return
+
+                    user= parts[1]
+                    if user == self.user:
+                        self.pInfo( _('You can\'t talk to yourself') )
+                        return
+                    if not "@" in user:
+                        self.pError( _('You must use give the full passport address. Ex.: somebody@example.com') )
+                        return
+
+                    chat= self.newChat( user )
+                    self.sortedChats.append( chat )
+
+                    self.activeChat= chat
+                    self.Main.newWin( chat )
+                    self.refreshWin()
+                    self.showChatBar()
+                    self.showStateBar()
             
             elif command == 'n':
                 if len(parts) == 1: 
@@ -1054,8 +1114,8 @@ class TextMSN(MSN.Session):
 
             elif command == '!':
                 com= join( parts[1:] )
-                out= commands.getoutput( com )
-                self.pActive( out + '\n' )
+                (stdin, out)= os.popen4(com, 'r')
+                self.pActive( out.read() + '\n' )
 
             elif command == 'e':
                 self.Main.erase( self.activeChat )
@@ -1073,6 +1133,12 @@ class TextMSN(MSN.Session):
                 self.subExec( self.replaceVars( 
                     'echo $user es una $prueba'
                     , {'$user':'de aqui', '$prueba':'kk' } ) )
+            
+	    ## added for pebrot no error message shutdown
+	    elif command == 'rec':
+	        if self.offline == 1:
+                    self.crash()
+		    self.offline = 0
 
             elif command == 'd':
                 if len( parts ) == 1:
@@ -1088,7 +1154,15 @@ class TextMSN(MSN.Session):
                 self.Main.dump( self.activeChat, f )
                 self.pInfo( _('Chat dumped to file %s') % name )
 
-            elif command == 'q':
+            elif command == 'color':
+	        if len( parts ) == 1:
+		    self.pInfo( _('The colors are: %s') % self.COLORS )
+		elif len ( parts ) == 3:
+		    #self.pInfo( _('%s') % parts[1] + ' and ' + parts[2] )
+		    self.COLORS[ parts[1] ] = self.COLORS[ parts[2] ]
+		else:
+		     self.pInfo( _('You messed up somehow and i can\'t fix it. :-P') )
+	    elif command == 'q':
                 sys.exit()
             else:
                 self.pError( _(r'Error: Unknown command: %s') % parts[0] )
@@ -1469,9 +1543,11 @@ class TextMSN(MSN.Session):
             else:
                 type= self.socketType( e.socket )
                 if type == MSN.SOCK_MAIN:
-                    self.pError( _('Socket error on main server connection (%s)') % e.errorMsg() )
-                    self.pError( _('Trying to reconnect...') )
-                    self.reconnect()
+		    # Added to Fix offline status
+		    if self.state != 'FLN':
+                        self.pError( _('Socket error on main server connection (%s)') % e.errorMsg() )
+                        self.pError( _('Trying to reconnect...') )
+                        self.reconnect()
                 elif type == MSN.SOCK_RECV:
                     fRecv= self.recvFile[e.socket] 
                     self.pError( _('Socket error receiving \'%s\' (%s)') 
@@ -1485,8 +1561,10 @@ class TextMSN(MSN.Session):
                     self.removeFileSend( fSend )
                     self.showChatBar()
                 elif type == MSN.SOCK_CHAT:
-                    self.pError( _('Socket error (%s)') % e.errorMsg(), 
-                        self.chats[e.socket] )
+                    # To hide errors when server closes idle chat
+                    if not self.chats[e.socket].hideSocketError:
+                        self.pError( _('Socket error (%s)') % e.errorMsg(), 
+                            self.chats[e.socket] )
                 else:
                     self.pError( _('Unhandled socket error (%s)') 
                         % e.errorMsg() )
@@ -1534,6 +1612,14 @@ class TextMSN(MSN.Session):
         return chat
 
     def drawLastMove( self, chat, msg ):
+        allUsers= self.usersList.keys()
+        allUsers.sort()
+        number= allUsers.index( msg.fields['passport'] )
+
+        self.pWin( chat, '%s' % number ,'LIST_NUM', 0 )
+        self.pWin( chat, '- ', 'DEFAULT', 0 )
+
+
         self.pWin( chat, '%s ' % msg.fields['passport']
             , 'LIST_PASS', 0 )
         self.pWin( chat, _('changed state to '), 'DEFAULT', 0 )
@@ -1586,6 +1672,11 @@ class TextMSN(MSN.Session):
     def userOn( self, passport ):
         self.updateUsersList()
 
+        # Lets reshow user list on init when an ILN arrives after it wasn't expected 
+        if self.listShown:
+            chat = self.getAutoListChat()
+            self.printUsers( chat ) 
+
     def userOff( self, passport ):
         self.updateUsersList()
 
@@ -1603,7 +1694,11 @@ class TextMSN(MSN.Session):
                 , '$user':escName} )
 
     def otherLogin( self ):
-        self.fatalError( _('Detected other login to this account') )
+        #self.fatalError( _('Detected other login to this account') )
+  	    self.pError( _('You have been signed in from another computer. Type \\q to exit or \\s nln to reconnect.') )
+	    self.state = 'FLN'
+	    self.pError( _('You are currently OFFLINE.') )
+	
     
     def userAdded( self, passport, list ):
         list= upper( list )
@@ -1628,6 +1723,8 @@ class TextMSN(MSN.Session):
 
     def msgFrom( self, chat, passport, msg ):
         # Make chat visible on first msg received
+        self.pDebug(passport)
+        
         if chat not in self.sortedChats:
             self.pInfo( _('%s is talking to you') % passport )
             if BEEP_ON_NEW_CHAT:
@@ -1640,7 +1737,17 @@ class TextMSN(MSN.Session):
 
             if SHOW_INVITED:
                 self.sortedChats.append( chat )
-                self.Main.newWin( chat )
+                
+                # Lets try to reuse a chat window 
+                oldChat = self.Main.reuseWindow( chat )
+                if oldChat:
+                    self.delChat( oldChat )
+                    self.sortedChats.remove( oldChat )
+                    if self.activeChat == oldChat:
+                        self.activeChat = chat
+                    self.refreshWin()    
+                else:    
+                    self.Main.newWin( chat )
 
         # Avoid problems displaying \r
         msg= replace( msg, '\r', '' )
@@ -1656,8 +1763,13 @@ class TextMSN(MSN.Session):
     def gotMail( self, numUnread ):
         self.pInfo( _('You\'ve got mail: %s unread.') % numUnread )
 
-    def incomingMail( self, mailFrom ):
-        self.pInfo( _('Incoming mail from: %s') % mailFrom )
+    def incomingMail( self, mailFrom, nameFrom ):
+        if EXEC_ON_NEW_MAIL:
+            escMail= re.escape( mailFrom )
+            escName= re.escape( nameFrom )
+            self.execAction( EXEC_ON_NEW_MAIL, {'$mailfrom':escMail, '$namefrom':escName} )
+          
+        self.pInfo( _('Incoming mail from: %s <%s>') % (mailFrom, nameFrom) )
 
     def userAcceptChat( self, chat, passport ):
         if not self.Main.hasWin( chat ):
