@@ -103,69 +103,70 @@ MSGS= {
          'FIL':['size'],
          'QNG':[],
          'NOT':['size'],
+         'UUX':['id', 'length'],
        }
 
 ERRORS= {      
-	        200: _('Invalid syntax'),
-	        201: _('Invalid parameter'),
-	        205: _('Invalid user'),
-	        206: _('Domain name missing'),
-	        207: _('Already logged in'),
-	        208: _('Invalid username'),
-	        209: _('Nickname change illegal'),
-	        210: _('User list full'),
-	        215: _('User already on list'),
-	        216: _('User already not on list'),
-	        217: _('User not online'),
-	        218: _('Already in mode'),
-	        219: _('User is in the opposite list'),
-	        223: _('Too many groups'),
-	        224: _('Invalid group'),
-	        225: _('User not in group'),
-	        229: _('Group name too long'),
-	        230: _('Cannot remove group zero'),
-	        231: _('Invalid group'),
-	        280: _('Switchboard failed'),
-	        281: _('Transfer to switchboard failed'),
-	        300: _('Required field missing'),
-	        301: _('Too many hits to a FND'),
-	        302: _('Not logged in'),
-	        500: _('Internal server error'),
-	        501: _('Database server error'),
-	        502: _('Command disabled'),
-	        510: _('File operation failed'),
-	        520: _('Memory allocation failed'),
+            200: _('Invalid syntax'),
+            201: _('Invalid parameter'),
+            205: _('Invalid user'),
+            206: _('Domain name missing'),
+            207: _('Already logged in'),
+            208: _('Invalid username'),
+            209: _('Nickname change illegal'),
+            210: _('User list full'),
+            215: _('User already on list'),
+            216: _('User already not on list'),
+            217: _('User not online'),
+            218: _('Already in mode'),
+            219: _('User is in the opposite list'),
+            223: _('Too many groups'),
+            224: _('Invalid group'),
+            225: _('User not in group'),
+            229: _('Group name too long'),
+            230: _('Cannot remove group zero'),
+            231: _('Invalid group'),
+            280: _('Switchboard failed'),
+            281: _('Transfer to switchboard failed'),
+            300: _('Required field missing'),
+            301: _('Too many hits to a FND'),
+            302: _('Not logged in'),
+            500: _('Internal server error'),
+            501: _('Database server error'),
+            502: _('Command disabled'),
+            510: _('File operation failed'),
+            520: _('Memory allocation failed'),
             540: _('Challenge response failed'),
-	        600: _('Server is busy'),
-	        601: _('Server is unavaliable'),
-	        602: _('Peer nameserver is down'),
-	        603: _('Database connection failed'),
-	        604: _('Server is going down'),
-	        605: _('Server unavaliable'),
-	        707: _('Could not create connection'),
+            600: _('Server is busy'),
+            601: _('Server is unavaliable'),
+            602: _('Peer nameserver is down'),
+            603: _('Database connection failed'),
+            604: _('Server is going down'),
+            605: _('Server unavaliable'),
+            707: _('Could not create connection'),
             710: _('Bad CVR parameters sent'),
-	        711: _('Write is blocking'),
-	        712: _('Session is overloaded'),
-	        713: _('Too many active users'),
-	        714: _('Too many sessions'),
-	        715: _('Not expected'),
-	        717: _('Bad friend file'),
-	        731: _('Not expected'),
-	        800: _('Friendly name changes too rapidly'),
-	        910: _('Server too busy'),
-	        911: _('Authentication failed'),
-	        912: _('Authentication failed'),
-	        913: _('Not allowed when offline'),
-	        914: _('Server unavailable'),
-	        915: _('Server unavailable'),
-	        916: _('Server unavailable'),
-	        917: _('Authentication failed'),
-	        918: _('Server too busy'),
-	        919: _('Server too busy'),
-	        920: _('Not accepting new users'),
-	        921: _('Server too busy'),
-	        922: _('Server too busy'),
-	        923: _('Kids Passport without parental consent'),
+            711: _('Write is blocking'),
+            712: _('Session is overloaded'),
+            713: _('Too many active users'),
+            714: _('Too many sessions'),
+            715: _('Not expected'),
+            717: _('Bad friend file'),
+            731: _('Not expected'),
+            800: _('Friendly name changes too rapidly'),
+            910: _('Server too busy'),
+            911: _('Authentication failed'),
+            912: _('Authentication failed'),
+            913: _('Not allowed when offline'),
+            914: _('Server unavailable'),
+            915: _('Server unavailable'),
+            916: _('Server unavailable'),
+            917: _('Authentication failed'),
+            918: _('Server too busy'),
+            919: _('Server too busy'),
+            920: _('Not accepting new users'),
+            921: _('Server too busy'),
+            922: _('Server too busy'),
+            923: _('Kids Passport without parental consent'),
             924: _('Passport account not yet verified'),
         }
 
@@ -510,14 +511,18 @@ class Msg(ExParrot):
             raise MSNError, (self.type, int(parts[1]) )
         elif ( self.type not in ['GTC', 'BPR'] ):
             maxLen= len( parts )
-            expLen= len( MSGS[self.type] )+1
-            if ( expLen < maxLen ):
-                maxLen= expLen
+            if self.type in MSGS:
+                expLen= len( MSGS[self.type] )+1
+                if ( expLen < maxLen ):
+                    maxLen= expLen
 
-            for i in range( 1, maxLen ):
-                self.fields[MSGS[self.type][i-1]]= parts[i]
-            
-            
+                for i in range( 1, maxLen ):
+                    self.fields[MSGS[self.type][i-1]]= parts[i]
+            else:        
+                self.pDebug( 'Unknown message type: %s' % (self.type) )
+                
+                
+                
     def __str__( self ):        
         return self.st
         
@@ -694,6 +699,9 @@ class Session( Comm ):
             self.removeFileRecv( fRecv )
 
         chat.send( 'OUT', '\r\n' )
+        self.delChat(chat)
+
+    def delChat( self, chat ):
         del( self.chats[chat.s] )
         self.delConnectingChat( chat )
         self.inputs.remove( chat.s )
@@ -713,12 +721,12 @@ class Session( Comm ):
 
     def changeState( self, state ):
         ## Added to allow Pebrot to stay alive offline
-	if state == "FLN":
+        if state == "FLN":
             self.send ( 'OUT', '\r\n' )
             self.state = 'FLN'
             self.pInfo( 'You have been logged out. Type \\q to exit or \\s nln to reconnect.' )
-	else:
-	    self.send( 'CHG', '%s\r\n' % state )
+        else:
+            self.send( 'CHG', '%s\r\n' % state )
 
     def challenge( self, msg ):
         chunk= md5.new( msg.fields['hash']+ 'Q1P7W2E4J9R8U3S5' ).hexdigest()
@@ -784,9 +792,9 @@ class Session( Comm ):
             self.delUserError( info[0], info[1] )
         elif exc.code in [205, 209, 217, 219, 224, 231, 500]:
             self.protoError( exc.code )
-	#fix crash when adding invalid usernames
-	elif exc.code == 208:
-	    self.pError( _('The user may not be added because no account exists by that name.') )
+        #fix crash when adding invalid usernames
+        elif exc.code == 208:
+            self.pError( _('The user may not be added because no account exists by that name.') )
         else:
             self.fatalProtoError( exc.code )
 
@@ -804,9 +812,9 @@ class Session( Comm ):
                 self.gotMail( m.group(1) )        
         elif type == 'x-msmsgsemailnotification':
             if find( text, 'From:') != -1:
-                m= re.compile(r'From: *(.+)\r\n').search( text )
-                if m:
-                    self.incomingMail( m.group(1) )        
+                mail= re.compile(r'From: *(.+)\r\n').search( text )
+                name= re.compile(r'From-Addr: *(.+)\r\n').search( text )
+                self.incomingMail( mail.group(1), name.group(1) )        
 
     def addFileSend( self, fSend ):
         self.sendFile[fSend.s]= fSend
@@ -1144,9 +1152,13 @@ class Session( Comm ):
                 name= name.encode( ENCODING, 'replace' )
                 self.nameChanged( name )
             elif ( msg.type == 'SYN' ):
-                self.lstTotal= int( msg.fields['numPeople'] )
-                self.lstNum= 0
-                self.pDebug( 'lstTotal= %d' % self.lstTotal )
+                if msg.fields.has_key('numPeople'): # Ok, we are gonna receive some LSTs 
+                    self.lstTotal= int( msg.fields['numPeople'] )
+                    self.lstNum= 0
+                    self.pDebug( 'lstTotal= %d' % self.lstTotal )
+                #else: # List is up-to-date 
+                    #self.changeState( self.state )
+                    #self.ping()
             elif ( msg.type == 'LST' ):
                 listNums= int( msg.fields['lists'] )
                 if listNums == 8:
@@ -1461,7 +1473,7 @@ class Session( Comm ):
     def gotMail( self, numUnread ):
         self.pDebug( 'gotMail -> NO CALLBACK' )
         
-    def incomingMail( self, mailFrom ):
+    def incomingMail( self, mailFrom, nameFrom ):
         self.pDebug( 'incomingMail -> NO CALLBACK' )
         
     def protoError( self, code ):
